@@ -10,12 +10,12 @@ use crate::{
     },
     double_spawn::DoubleSpawnInfo,
     errors::{
-        ConfigureHandleInheritanceError, StressCountParseError, TestRunnerBuildError,
-        TestRunnerExecuteErrors,
+        ConfigureHandleInheritanceError, DebuggerCommandParseError, StressCountParseError,
+        TestRunnerBuildError, TestRunnerExecuteErrors,
     },
     input::{InputHandler, InputHandlerKind, InputHandlerStatus},
     list::{TestInstanceWithSettings, TestList},
-    reporter::events::{RunStats, StressIndex, TestEvent},
+    reporter::events::{ReporterEvent, RunStats, StressIndex},
     runner::ExecutorEvent,
     signal::{SignalHandler, SignalHandlerKind},
     target_runner::TargetRunner,
@@ -36,6 +36,70 @@ use tokio::{
 };
 use tracing::{debug, warn};
 
+/// A parsed debugger command.
+#[derive(Clone, Debug)]
+pub struct DebuggerCommand {
+    program: String,
+    args: Vec<String>,
+}
+
+impl DebuggerCommand {
+    /// Gets the program.
+    pub fn program(&self) -> &str {
+        // The from_str constructor ensures that there is at least one part.
+        &self.program
+    }
+
+    /// Gets the arguments.
+    pub fn args(&self) -> &[String] {
+        &self.args
+    }
+}
+
+impl FromStr for DebuggerCommand {
+    type Err = DebuggerCommandParseError;
+
+    fn from_str(command: &str) -> Result<Self, Self::Err> {
+        let mut parts =
+            shell_words::split(command).map_err(DebuggerCommandParseError::ShellWordsParse)?;
+        if parts.is_empty() {
+            return Err(DebuggerCommandParseError::EmptyCommand);
+        }
+        let program = parts.remove(0);
+        Ok(Self {
+            program,
+            args: parts,
+        })
+    }
+}
+
+/// A child process identifier: either a single process or a process group.
+#[derive(Copy, Clone, Debug)]
+pub(super) enum ChildPid {
+    /// A single process ID.
+    Process(#[cfg_attr(not(unix), expect(unused))] u32),
+
+    /// A process group ID.
+    #[cfg(unix)]
+    ProcessGroup(u32),
+}
+
+impl ChildPid {
+    /// Returns the PID value to use with `libc::kill`.
+    ///
+    /// - `Process(pid)` returns `pid as i32` (positive, kills single process).
+    /// - `ProcessGroup(pid)` returns `-(pid as i32)` (negative, kills process group).
+    ///
+    /// On Windows, always returns `pid as i32`.
+    #[cfg(unix)]
+    pub(super) fn for_kill(self) -> i32 {
+        match self {
+            ChildPid::Process(pid) => pid as i32,
+            ChildPid::ProcessGroup(pid) => -(pid as i32),
+        }
+    }
+}
+
 /// Test runner options.
 #[derive(Debug, Default)]
 pub struct TestRunnerBuilder {
@@ -44,6 +108,7 @@ pub struct TestRunnerBuilder {
     max_fail: Option<MaxFail>,
     test_threads: Option<TestThreads>,
     stress_condition: Option<StressCondition>,
+    debugger: Option<DebuggerCommand>,
 }
 
 impl TestRunnerBuilder {
@@ -86,6 +151,12 @@ impl TestRunnerBuilder {
         self
     }
 
+    /// Sets the debugger to use for running tests.
+    pub fn set_debugger(&mut self, debugger: DebuggerCommand) -> &mut Self {
+        self.debugger = Some(debugger);
+        self
+    }
+
     /// Creates a new test runner.
     #[expect(clippy::too_many_arguments)]
     pub fn build<'a>(
@@ -132,6 +203,7 @@ impl TestRunnerBuilder {
                 cli_args,
                 max_fail,
                 stress_condition: self.stress_condition,
+                debugger: self.debugger,
                 runtime,
             },
             signal_handler,
@@ -201,10 +273,10 @@ impl<'a> TestRunner<'a> {
         mut callback: F,
     ) -> Result<RunStats, TestRunnerExecuteErrors<Infallible>>
     where
-        F: FnMut(TestEvent<'a>) + Send,
+        F: FnMut(ReporterEvent<'a>) + Send,
     {
-        self.try_execute::<Infallible, _>(|test_event| {
-            callback(test_event);
+        self.try_execute::<Infallible, _>(|event| {
+            callback(event);
             Ok(())
         })
     }
@@ -220,7 +292,7 @@ impl<'a> TestRunner<'a> {
         mut callback: F,
     ) -> Result<RunStats, TestRunnerExecuteErrors<E>>
     where
-        F: FnMut(TestEvent<'a>) -> Result<(), E> + Send,
+        F: FnMut(ReporterEvent<'a>) -> Result<(), E> + Send,
         E: fmt::Debug + Send,
     {
         let (report_cancel_tx, report_cancel_rx) = oneshot::channel();
@@ -283,6 +355,7 @@ struct TestRunnerInner<'a> {
     cli_args: Vec<String>,
     max_fail: MaxFail,
     stress_condition: Option<StressCondition>,
+    debugger: Option<DebuggerCommand>,
     runtime: Runtime,
 }
 
@@ -295,10 +368,17 @@ impl<'a> TestRunnerInner<'a> {
         callback: F,
     ) -> Result<RunStats, Vec<JoinError>>
     where
-        F: FnMut(TestEvent<'a>) + Send,
+        F: FnMut(ReporterEvent<'a>) + Send,
     {
         // TODO: add support for other test-running approaches, measure performance.
 
+        // Disable the global timeout when a debugger is active.
+        let global_timeout = if self.debugger.is_some() {
+            crate::time::far_future_duration()
+        } else {
+            self.profile.global_timeout().period
+        };
+
         let mut dispatcher_cx = DispatcherContext::new(
             callback,
             self.run_id,
@@ -306,7 +386,7 @@ impl<'a> TestRunnerInner<'a> {
             self.cli_args.clone(),
             self.test_list.run_count(),
             self.max_fail,
-            self.profile.global_timeout().period,
+            global_timeout,
             self.stress_condition.clone(),
         );
 
@@ -318,6 +398,7 @@ impl<'a> TestRunnerInner<'a> {
             self.target_runner.clone(),
             self.capture_strategy,
             self.force_retries,
+            self.debugger.clone(),
         );
 
         // Send the initial event.
@@ -380,7 +461,7 @@ impl<'a> TestRunnerInner<'a> {
         report_cancel_rx: Pin<&mut Fuse<oneshot::Receiver<()>>>,
     ) -> Result<(), Vec<JoinError>>
     where
-        F: FnMut(TestEvent<'a>) + Send,
+        F: FnMut(ReporterEvent<'a>) + Send,
     {
         let ((), results) = TokioScope::scope_and_block(move |scope| {
             let (resp_tx, resp_rx) = unbounded_channel::<ExecutorEvent<'a>>();
@@ -614,4 +695,29 @@ mod tests {
         assert_eq!(runner.inner.capture_strategy, CaptureStrategy::None);
         assert_eq!(runner.inner.test_threads, 1, "tests run serially");
     }
+
+    #[test]
+    fn test_debugger_command_parsing() {
+        // Valid commands
+        let cmd = DebuggerCommand::from_str("gdb --args").unwrap();
+        assert_eq!(cmd.program(), "gdb");
+        assert_eq!(cmd.args(), &["--args"]);
+
+        let cmd = DebuggerCommand::from_str("rust-gdb -ex run --args").unwrap();
+        assert_eq!(cmd.program(), "rust-gdb");
+        assert_eq!(cmd.args(), &["-ex", "run", "--args"]);
+
+        // With quotes
+        let cmd = DebuggerCommand::from_str(r#"gdb -ex "set print pretty on" --args"#).unwrap();
+        assert_eq!(cmd.program(), "gdb");
+        assert_eq!(cmd.args(), &["-ex", "set print pretty on", "--args"]);
+
+        // Empty command
+        let err = DebuggerCommand::from_str("").unwrap_err();
+        assert!(matches!(err, DebuggerCommandParseError::EmptyCommand));
+
+        // Whitespace only
+        let err = DebuggerCommand::from_str("   ").unwrap_err();
+        assert!(matches!(err, DebuggerCommandParseError::EmptyCommand));
+    }
 }
